//////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2014 UTIA AV CR v.v.i. All rights reserved.
//
// UTIA IS PROVIDING THIS CODE, "AS IS" AS A COURTESY TO YOU.
// BY PROVIDING THIS CODE, AS ONE POSSIBLE IMPLEMENTATION,
// UTIA IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
// IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
// FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
// UTIA EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
// THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
// ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
// FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.
//
//////////////////////////////////////////////////////////////////////////

#define UTIASPEDK_FPGA_3S700AN (1)

#include "xparameters.h"
#include "xuartlite.h"
#include "stdio.h"
#include "string.h"

#include "wal.h"
#include "wal_bce_jk.h"

#ifdef SG_PLBIFACE_COSIM
#include "sg_plbiface.h"

WAL_REGISTER_WORKER(worker1, BCE_JK_FP32M24, SG_PLBIFACE_ConfigTable, 0, 1, 0);
#else
#include "bce_fp11_1x1_0_plbw.h"

WAL_REGISTER_WORKER(worker1, BCE_JK_FP32M24, BCE_FP11_1X1_0_PLBW_ConfigTable, 0,
		1, 0);
#endif

#include "fill_FA1101P0_program_store.h"
#include "fill_FA1101P1_program_store.h"

#include "fill_FP1101P0_program_store.h"
#include "fill_FP1101P1_program_store.h"

#if SG_PLBIFACE_COSIM
#define NN  (10)
#else
#define NN  (10)
#endif

#define ERR (0.0f)

float af1[NN], bf1[NN], zf1[NN];
float af1r[NN], bf1r[NN], zf1r[NN];

void draw_pb2mb_introduction(wal_worker_t *wrk) {
	char buf[32];
	uint32_t status_led, status_screen;

	wal_pb2mb(wrk, &status_screen);
	while ((status_screen & 0x7) != 0) {

		strcpy(buf, "\r\n Screen0 ");
		xil_printf("\r\n Screen");
		buf[9] = (char) ((status_screen & 0xFF) + 0x30);
		xil_printf("%c\r\n", (char) buf[9]);
		//mfs_file_write(fw, buf, 10);

		wal_pb2mb(wrk, &status_led);
		while ((status_led & WAL_STAT_EXIT_BIT)== 0){
			buf[0] = (char) (status_led & 0xFF);
			xil_printf("%c", (char) buf[0]);
			//mfs_file_write(fw, buf, 1);

			wal_pb2mb(wrk, &status_led);
		}
		wal_pb2mb(wrk, &status_screen);
	}
	strcpy(buf, "\r\n Screen0 ");
	xil_printf("\r\n Screen");
	buf[9] = (char) ((status_screen & 0xFF) + 0x30);
	xil_printf("%c\r\n", (char) buf[9]);
	//mfs_file_write(fw, buf, 10);
	wal_pb2mb(wrk, &status_led);
	while ((status_led & WAL_STAT_EXIT_BIT)== 0){
		buf[0] = (char) (status_led & 0xFF);
		xil_printf("%c", (char) buf[0]);
		//mfs_file_write(fw, buf, 1);
		wal_pb2mb(wrk, &status_led);
	}
}


float vprod_mb(unsigned int nn, float * afmb, float * bfmb) {
	unsigned int i, j;
	float zs[4];

	for (i = 0; i < 4; i++) {
		zs[i] = 0;
	}

	j = 0;
	for (i = 0; i < nn; i++) {
		zs[j] += afmb[i] * bfmb[i];
		if (j < 3)
			j = j + 1;
		else
			j = 0;
	}
	zs[0] += zs[1];
	zs[2] += zs[3];
	zs[0] += zs[2];

	return ((float) zs[0]); //vector product
}

void vmac_mb(unsigned int nn, float * afmb, float * bfmb, float * zfmb) {
	unsigned int i;
	for (i = 0; i < nn; i++) {
		zfmb[i] += afmb[i] * bfmb[i];
	}
}

void vmsubac_mb(unsigned int nn, float * afmb, float * bfmb, float * zfmb) {
	unsigned int i;
	for (i = 0; i < nn; i++) {
		zfmb[i] -= afmb[i] * bfmb[i];
	}
}


void bce_fp11_1x1_0_VZ2A(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {

	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;

	//====================================================
	if (capab & WAL_BCE_JK_CAP_VZ2A) {
		nn = NN;
		xil_printf("\r\n Test VZ2A '%s' .............", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VZ2A;
		for (i = 0; i < nn; i++) {
			zf1[i] = (float) i + (float) 0.123456;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			if (af1r[i] != zf1[i]) {
				xil_printf("\r\n VZ2A Error: af1r[%d] = %d", i,
						(unsigned int) af1r[i]);
				j++;
			}
		}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VZ2A is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VB2A(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {

	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VB2A) {
		nn = NN;
		xil_printf("\r\n Test VB2A '%s' .............", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VB2A;
		for (i = 0; i < nn; i++) {
			bf1[i] = (float) i + (float) 0.123456;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			if (af1r[i] != bf1[i]) {
				xil_printf("\r\n VB2A Error: af1r[%d] = %d", i,
						(unsigned int) af1r[i]);
				j++;
			}
		}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VB2A is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VZ2B(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {

	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VZ2B) {
		nn = NN;
		xil_printf("\r\n Test VZ2B '%s' .............", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VZ2B;
		for (i = 0; i < nn; i++) {
			zf1[i] = (float) i + (float) 1.23456;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			if (bf1r[i] != zf1[i]) {
				xil_printf("\r\n VZ2B Error: bf1r[%d] = %d", i,
						(unsigned int) bf1r[i]);
				j++;
			}
		}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VZ2B is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VA2B(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {

	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VA2B) {
		nn = NN;
		xil_printf("\r\n Test VA2B '%s' .............", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VA2B;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.23456;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			if (bf1r[i] != af1[i]) {
				xil_printf("\r\n VA2B Error: bf1r[%d] = %d", i,
						(unsigned int) bf1r[i]);
				j++;
			}
		}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VA2B is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VADD(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VADD) {
		nn = NN;
		xil_printf("\r\n Test VADD '%s' .............", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VADD;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 0.123456;
			bf1[i] = (float) i + (float) 0.789;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] + bf1[i];
			err = zf1r[i] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VADD Error: zf1r[%d] = %d   r = %d",i,(unsigned int) zf1r[i], (unsigned int) r);
			j++;
		}
	}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VADD is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VADD_BZ2A(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VADD_BZ2A) {
		nn = NN;
		xil_printf("\r\n Test VADD_BZ2A '%s' ........", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VADD_BZ2A;
		for (i = 0; i < nn; i++) {
			bf1[i] = (float) i + (float) 1.23456;
			zf1[i] = (float) i + (float) 1.789;
			af1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = bf1[i] + zf1[i];
			err = af1r[i] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VADD_BZ2A Error: af1r[%d] = %d   r = %d",i,(unsigned int) af1r[i], (unsigned int) r);
			j++;
		}
	}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VADD_BZ2A is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VADD_AZ2B(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VADD_AZ2B) {
		nn = NN;
		xil_printf("\r\n Test VADD_AZ2B '%s' ........", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VADD_AZ2B;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.12345;
			zf1[i] = (float) i + (float) 1.23456;
			bf1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] + zf1[i];
			err = bf1r[i] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VADD_AZ2B Error: bf1r[%d] = %d   r = %d",i,(unsigned int) bf1r[i], (unsigned int) r);
			j++;
		}
	}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VADD_AZ2B is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VSUB(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VSUB) {
		nn = NN;
		xil_printf("\r\n Test VSUB '%s' .............", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VSUB;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 2.23456;
			bf1[i] = (float) 1.123456;
			zf1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] - bf1[i];
			err = zf1r[i] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VSUB Error: zf1r[%d] = %d   r = %d",i,(unsigned int) zf1r[i], (unsigned int) r);
			j++;
		}
	}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VSUB is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VSUB_BZ2A(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VSUB_BZ2A) {
		nn = NN;
		xil_printf("\r\n Test VSUB_BZ2A '%s' ........", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VSUB_BZ2A;
		for (i = 0; i < nn; i++) {
			bf1[i] = (float) i + (float) 2.12345;
			zf1[i] = (float) 1.6789;
			af1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = bf1[i] - zf1[i];
			err = af1r[i] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VSUB_BZ2A Error: af1r[%d] = %d   r = %d",i,(unsigned int) af1r[i], (unsigned int) r);
			j++;
		}
	}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VSUB_BZ2A is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VSUB_AZ2B(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VSUB_AZ2B) {
		nn = NN;
		xil_printf("\r\n Test VSUB_AZ2B '%s' ........", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VSUB_AZ2B;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.123456;
			zf1[i] = (float) i + (float) 1.6789;
			bf1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] - zf1[i];
			err = bf1r[i] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VSUB_AZ2B Error: bf1r[%d] = %d   r = %d",i,(unsigned int) bf1r[i], (unsigned int) r);
			j++;
		}
	}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VSUB_AZ2B is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VMULT(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMULT) {
		nn = NN;
		xil_printf("\r\n Test VMULT '%s' ............", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMULT;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.12345;
			bf1[i] = (float) i + (float) 0.76543;
			zf1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] * bf1[i];
			err = zf1r[i] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VMULT Error: zf1r[%d] = %d   r = %d",i,(unsigned int) zf1r[i], (unsigned int) r);
			j++;
		}
	}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VMULT is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VMULT_BZ2A(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMULT_BZ2A) {
		nn = NN;

		xil_printf("\r\n Test VMULT_BZ2A '%s' .......", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMULT_BZ2A;
		for (i = 0; i < nn; i++) {
			bf1[i] = (float) i + (float) 1.23456;
			zf1[i] = (float) i + (float) 0.76543;
			af1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = bf1[i] * zf1[i];
			err = af1r[i] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VMULT_BZ2A Error: af1r[%d] = %d   r = %d",i,(unsigned int) af1r[i], (unsigned int) r);
			j++;
		}
	}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VMULT_BZ2A is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VMULT_AZ2B(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMULT_AZ2B) {
		nn = NN;
		xil_printf("\r\n Test VMULT_AZ2B '%s' .......", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMULT_AZ2B;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.12345;
			zf1[i] = (float) i + (float) 0.76543;
			bf1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] * zf1[i];
			err = bf1r[i] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VMULT_AZ2B Error: bf1r[%d] = %d   r = %d",i,(unsigned int) bf1r[i], (unsigned int) r);
			j++;
		}
	}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VMULT_AZ2B is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VPROD(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VPROD) {
		nn = NN;
		xil_printf("\r\n Test VPROD '%s' ............", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 0;
		op = WAL_BCE_JK_VPROD;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.123456;
			bf1[i] = (float) i + (float) 0.76543;
			zf1[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		j = 0;
		r = vprod_mb(nn, af1, bf1);
		err = zf1r[0] - r;
		if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
		xil_printf("\r\n VPROD Error: zf1r[0] = %d   r = %d",(unsigned int) zf1r[0], (unsigned int) r);
		j++;
	}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VPROD is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VMAC(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	float err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMAC) {
		if (NN > 10) {
			nn = 10;
		} else {
			nn = NN;
		}
		xil_printf("\r\n Test VMAC '%s' .............", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMAC;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.123456;
			bf1[i] = (float) i + (float) 0.76543;
			zf1[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		vmac_mb(nn, af1, bf1, zf1);

		j = 0;
		for (i = 0; i < nn; i++) {
			err = zf1[i] - zf1r[i];
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VMAC Error: zf1[%d] = %d   zf1r[%d] = %d",i,(unsigned int) zf1[i], i, (unsigned int) zf1r[i]);
			j++;
		}
	}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VMAC is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VMSUBAC(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMSUBAC) {
		if (NN > 10) {
			nn = 10;
		} else {
			nn = NN;
		}
		xil_printf("\r\n Test VMSUBAC '%s' ..........", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMSUBAC;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.123456;
			bf1[i] = (float) i + (float) 0.76543;
			zf1[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		vmsubac_mb(nn, af1, bf1, zf1);

		j = 0;
		for (i = 0; i < nn; i++) {
			err = zf1[i] - zf1r[i];
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VMSUBAC Error: zf1[%d] = %d   zf1r[%d] = %d",i,(unsigned int) zf1[i], i, (unsigned int) zf1r[i]);
			j++;
		}
	}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VMSUBAC is not supported.");
	}
	//====================================================
}

void bce_fp11_1x1_0_VDIV(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VDIV) {
		nn = NN;
		xil_printf("\r\n Test VDIV '%s' .............", wrk->name);
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VDIV;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 0.123456;
			bf1[i] = (float) i + (float) 0.789;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb_introduction(wrk);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] / bf1[i];
			err = zf1r[i] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VDIV Error: zf1r[%d] = %d   r = %d",i,(unsigned int) zf1r[i], (unsigned int) r);
			j++;
		}
	}
		if (j == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n VDIV is not supported.");
	}
	//====================================================
}

//////////////////////////////////////////////////////////////////////////
// DSP_PLATFORM (c) UTIA AV CR v.v.i. 2007-2010 
// main

int main(void) {
	unsigned int ah, bh, zh, i;
	unsigned int j, k;
	unsigned int capabilities1;
	unsigned int license1;

	/*
	 * Enable and initialize cache
	 */
#if XPAR_MICROBLAZE_0_USE_ICACHE
	microblaze_init_icache_range(0, XPAR_MICROBLAZE_0_CACHE_BYTE_SIZE);
	microblaze_enable_icache();
#endif

#if XPAR_MICROBLAZE_0_USE_DCACHE
	microblaze_init_dcache_range(0, XPAR_MICROBLAZE_0_DCACHE_BYTE_SIZE);
	microblaze_enable_dcache();
#endif

	if (wal_init_worker(worker1)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		return -1;
	}
	xil_printf("\r\n -- Entering main() --\r\n");
	xil_printf("\r\n Tests of vector operations.");

	///////////////////////////////////////////////
	// Upgrade firmware
	for (i = 0; i < 3; i++) {
		fill_FP1101P0_program_store[i] = fill_FA1101P0_program_store[i];
		fill_FP1101P1_program_store[i] = fill_FA1101P1_program_store[i];
	}
	for (i = EDKDSP_BOOT; i < 1024; i++) {
		fill_FP1101P0_program_store[i] = fill_FA1101P0_program_store[i];
		fill_FP1101P1_program_store[i] = fill_FA1101P1_program_store[i];
	}

	//Program and initalize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker1, WAL_PBID_P0, fill_FP1101P0_program_store, -1);
	wal_set_firmware(worker1, WAL_PBID_P1, fill_FP1101P1_program_store, -1);
	wal_get_capabilities(worker1, WAL_PBID_P0, &capabilities1);
	xil_printf("\r\n PB: HW Operations: Code = %x", capabilities1);

	//while (1) {
	//	 ________________________________________________________________
	//	|RGB565_CHAR1 Screen setup with 8 terminal windows             	 |
	//  |[RGB565_CHAR1_0][RGB565_CHAR1_1][RGB565_CHAR1_2][RGB565_CHAR1_3]|
	//	|[RGB565_CHAR1_4][RGB565_CHAR1_5][RGB565_CHAR1_6][RGB565_CHAR1_7]|
	//	|															     |
	//	|[UTIA]__________________________________________________________|
	//

	for (ah = 0; ah < 4; ah++) {
		for (bh = 0; bh < 4; bh++) {
			for (zh = 0; zh < 4; zh++) {
				// Test of the HW evaluation license
				wal_get_license(worker1, WAL_PBID_P0, &license1);
				if (license1) {
					xil_printf("\r\n\r\n ah=%d  bh=%d  zh=%d", ah, bh, zh);
					//bce_fp11_1x1_0_VZ2A(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VB2A(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VZ2B(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VA2B(worker1, capabilities1, ah, bh, zh);
					bce_fp11_1x1_0_VADD(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VADD_BZ2A(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VADD_AZ2B(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VSUB(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VSUB_BZ2A(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VSUB_AZ2B(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VMULT(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VMULT_BZ2A(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VMULT_AZ2B(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VPROD(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VMAC(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VMSUBAC(worker1, capabilities1, ah, bh, zh);
					//bce_fp11_1x1_0_VDIV(worker1, capabilities1, ah, bh, zh);
				} else {
					xil_printf("\r\n\r\n Evaluation license will expire soon. Please contact:");
					xil_printf("\r\n UTIA AV CR, v.v.i.; Department of Signal Processing");
					xil_printf("\r\n Pod vodarenskou vezi 4");
					xil_printf("\r\n 18208, Prague 8, Czech republic");
					xil_printf("\r\n http://zs.utia.cas.cz \r\n");
				}
			}
		}
	}

	xil_printf("\r\n\r\n-- Exiting main() --\r\n");
	/*
	 * Disable cache and reinitialize it so that other
	 * applications can be run with no problems
	 */
#if XPAR_MICROBLAZE_0_USE_DCACHE
	microblaze_disable_dcache();
	microblaze_init_dcache_range(0, XPAR_MICROBLAZE_0_DCACHE_BYTE_SIZE);
#endif

#if XPAR_MICROBLAZE_0_USE_ICACHE
	microblaze_disable_icache();
	microblaze_init_icache_range(0, XPAR_MICROBLAZE_0_CACHE_BYTE_SIZE);
#endif

	while (1) {
		asm("nop");
	}

	return 0;
}
