//////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2014 UTIA AV CR v.v.i. All rights reserved.
//
// UTIA IS PROVIDING THIS CODE, "AS IS" AS A COURTESY TO YOU.
// BY PROVIDING THIS CODE, AS ONE POSSIBLE IMPLEMENTATION,
// UTIA IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
// IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
// FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
// UTIA EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
// THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
// ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
// FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.
//
//////////////////////////////////////////////////////////////////////////

#include "xparameters.h"
#include "stdio.h"
#include "string.h"
#include "platform_config.h"
#include "platform_gpio.h"
#include "xilmfs.h"
#include "jpeg.h"
#include "wal.h"
#include "wal_bce_jk.h"
#if PRINT_RGB
#include "UtiaSpEdk_rgb565char.h"
#endif
#ifdef SG_PLBIFACE_COSIM
#include "sg_plbiface.h"
WAL_REGISTER_WORKER(worker1, BCE_JK_FP32M24, SG_PLBIFACE_ConfigTable, 0, 1, 0);
#else
#include "bce_fp11_1x1_0_plbw.h"

WAL_REGISTER_WORKER(worker1, BCE_JK_FP32M24, BCE_FP11_1X1_0_PLBW_ConfigTable, 0, 1, 0)
;
#endif

#include "fill_FA1101P0_program_store.h"
#include "fill_FA1101P1_program_store.h"


#include "fill_FP1101P0_program_store.h"
#include "fill_FP1101P1_program_store.h"

#if SG_PLBIFACE_COSIM
#define NN  (10)
#else
#define NN  (250)
#endif

#define ERR (0.0f)

float af1[NN], bf1[NN], zf1[NN];
float af1r[NN], bf1r[NN], zf1r[NN];

#if PRINT_RGB
#include "UtiaSpEdk_rgb565char.h"

RGB565_BITMAP RGB565_BITMAP1, RGB565_BITMAP2, RGB565_BITMAP3, RGB565_BITMAP4;
RGB565_BITMAP RGB565_BITMAP5, RGB565_BITMAP6, RGB565_BITMAP7, RGB565_BITMAP8;
RGB565_CHAR RGB565_CHAR1, RGB565_CHAR1b;
RGB565_CHAR RGB565_CHARUTIA, RGB565_CHARUTIAb;
RGB565_CHAR RGB565_CHAR1_0, RGB565_CHAR1b_0;
RGB565_CHAR RGB565_CHAR1_1, RGB565_CHAR1b_1;
RGB565_CHAR RGB565_CHAR1_2, RGB565_CHAR1b_2;
RGB565_CHAR RGB565_CHAR1_3, RGB565_CHAR1b_3;
RGB565_CHAR RGB565_CHAR1_4, RGB565_CHAR1b_4;
RGB565_CHAR RGB565_CHAR1_5, RGB565_CHAR1b_5;
RGB565_CHAR RGB565_CHAR1_6, RGB565_CHAR1b_6;
RGB565_CHAR RGB565_CHAR1_7, RGB565_CHAR1b_7;

int red, green, blue;
pixel_t clr;
#endif

void draw_accelerator_red_on(void) {
#if PRINT_RGB
	rectfill(&RGB565_BITMAP1, 27 * RGB565_CHAR_WIDTH,
			RGB565_Y - 14 * RGB565_CHAR_HEIGHT, 28 * RGB565_CHAR_WIDTH,
			RGB565_Y - 13 * RGB565_CHAR_HEIGHT, RGB565_00_00_31);
	rectfill(&RGB565_BITMAP1, 28 * RGB565_CHAR_WIDTH,
			RGB565_Y - 15 * RGB565_CHAR_HEIGHT + 2, 31 * RGB565_CHAR_WIDTH,
			RGB565_Y - 12 * RGB565_CHAR_HEIGHT - 2, RGB565_00_00_31);
#endif
}

void draw_accelerator_green_on(void) {
#if PRINT_RGB
	rectfill(&RGB565_BITMAP1, 27 * RGB565_CHAR_WIDTH,
			RGB565_Y - 14 * RGB565_CHAR_HEIGHT, 28 * RGB565_CHAR_WIDTH,
			RGB565_Y - 13 * RGB565_CHAR_HEIGHT, RGB565_00_63_00);
	rectfill(&RGB565_BITMAP1, 28 * RGB565_CHAR_WIDTH,
			RGB565_Y - 15 * RGB565_CHAR_HEIGHT + 2, 31 * RGB565_CHAR_WIDTH,
			RGB565_Y - 12 * RGB565_CHAR_HEIGHT - 2, RGB565_00_63_00);
#endif
}

void draw_accelerator_off(void) {
#if PRINT_RGB
	rectfill(&RGB565_BITMAP1, 27 * RGB565_CHAR_WIDTH,
			RGB565_Y - 14 * RGB565_CHAR_HEIGHT, 28 * RGB565_CHAR_WIDTH,
			RGB565_Y - 13 * RGB565_CHAR_HEIGHT, RGB565_03_07_03);
	rectfill(&RGB565_BITMAP1, 28 * RGB565_CHAR_WIDTH,
			RGB565_Y - 15 * RGB565_CHAR_HEIGHT + 2, 31 * RGB565_CHAR_WIDTH,
			RGB565_Y - 12 * RGB565_CHAR_HEIGHT - 2, RGB565_03_07_03);
#endif
}

void draw_fpu_on(void) {
#if PRINT_RGB
#if UTIASPEDK_FPGA_3S700AN
#else
	rectfill(&RGB565_BITMAP1, 20 * RGB565_CHAR_WIDTH,
			RGB565_Y - 15 * RGB565_CHAR_HEIGHT, 23 * RGB565_CHAR_WIDTH,
			RGB565_Y - 12 * RGB565_CHAR_HEIGHT, RGB565_31_63_31);
	rectfill(&RGB565_BITMAP1, 23 * RGB565_CHAR_WIDTH,
			RGB565_Y - 14 * RGB565_CHAR_HEIGHT, 24 * RGB565_CHAR_WIDTH,
			RGB565_Y - 13 * RGB565_CHAR_HEIGHT, RGB565_31_63_31);
#endif
#endif
}

void draw_fpu_off(void) {
#if PRINT_RGB
#if UTIASPEDK_FPGA_3S700AN
#else
	rectfill(&RGB565_BITMAP1, 20 * RGB565_CHAR_WIDTH,
			RGB565_Y - 15 * RGB565_CHAR_HEIGHT, 23 * RGB565_CHAR_WIDTH,
			RGB565_Y - 12 * RGB565_CHAR_HEIGHT, RGB565_03_07_03);
	rectfill(&RGB565_BITMAP1, 23 * RGB565_CHAR_WIDTH,
			RGB565_Y - 14 * RGB565_CHAR_HEIGHT, 24 * RGB565_CHAR_WIDTH,
			RGB565_Y - 13 * RGB565_CHAR_HEIGHT, RGB565_03_07_03);
#endif
#endif
}

void draw_pb2mb(wal_worker_t *wrk, int fw) {

	char buf[32];
	uint32_t status_led, status_screen;

	wal_pb2mb(wrk, &status_screen);
	while ((status_screen & 0x00000007) != 0) {

		strcpy(buf, "\r\n Screen0 ");
		buf[9] = (char) ((status_screen & 0x000000ff) + 0x30);
		mfs_file_write(fw, buf, 10);

		wal_pb2mb(wrk, &status_led);
		while ((status_led & WAL_STAT_EXIT_BIT) == 0) {
			buf[0] = (char) (status_led & 0x000000ff);
			mfs_file_write(fw, buf, 1);

#if PRINT_RGB
			if ((status_screen & 0x00000007) == 0x1)
				RGB565_CHAR_Write(&RGB565_BITMAP1, &RGB565_CHAR1_1,
						(unsigned char) (status_led & 0x000000ff));
			if ((status_screen & 0x00000007) == 0x2)
				RGB565_CHAR_Write(&RGB565_BITMAP1, &RGB565_CHAR1_2,
						(unsigned char) (status_led & 0x000000ff));
			if ((status_screen & 0x00000007) == 0x3)
				RGB565_CHAR_Write(&RGB565_BITMAP1, &RGB565_CHAR1_3,
						(unsigned char) (status_led & 0x000000ff));
			if ((status_screen & 0x00000007) == 0x4)
				RGB565_CHAR_Write(&RGB565_BITMAP1, &RGB565_CHAR1_4,
						(unsigned char) (status_led & 0x000000ff));
			if ((status_screen & 0x00000007) == 0x5)
				RGB565_CHAR_Write(&RGB565_BITMAP1, &RGB565_CHAR1_5,
						(unsigned char) (status_led & 0x000000ff));
			if ((status_screen & 0x00000007) == 0x6)
				RGB565_CHAR_Write(&RGB565_BITMAP1, &RGB565_CHAR1_6,
						(unsigned char) (status_led & 0x000000ff));
			if ((status_screen & 0x00000007) == 0x7)
				RGB565_CHAR_Write(&RGB565_BITMAP1, &RGB565_CHAR1_7,
						(unsigned char) (status_led & 0x000000ff));
			//if ((status_screen & 0x0000000B) == 0xB) RGB565_CHAR_Write (&RGB565_BITMAP1,&RGB565_CHAR1_11,(unsigned char) (status_led & 0x000000ff));
#endif
			wal_pb2mb(wrk, &status_led);
		}
		wal_pb2mb(wrk, &status_screen);
	}
	strcpy(buf, "\r\n Screen0 ");
	buf[9] = (char) ((status_screen & 0x000000ff) + 0x30);
	mfs_file_write(fw, buf, 10);
	wal_pb2mb(wrk, &status_led);
	while ((status_led & WAL_STAT_EXIT_BIT) == 0) {
		buf[0] = (char) (status_led & 0x000000ff);
		mfs_file_write(fw, buf, 1);
		wal_pb2mb(wrk, &status_led);
	}

#if PRINT_RGB
	if (status_led & 0x00000080) {
		rectfill(&RGB565_BITMAP1, 28 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 28 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_31_63_31);
	} else {
		rectfill(&RGB565_BITMAP1, 28 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 28 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_00_00_31);
	}
	if (status_led & 0x00000040) {
		rectfill(&RGB565_BITMAP1, 29 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 29 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_31_63_31);
	} else {
		rectfill(&RGB565_BITMAP1, 29 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 29 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_00_00_31);
	}

	if (status_led & 0x00000020) {
		rectfill(&RGB565_BITMAP1, 30 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 30 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_31_63_31);
	} else {
		rectfill(&RGB565_BITMAP1, 30 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 30 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_00_00_31);
	}

	if (status_led & 0x00000010) {
		rectfill(&RGB565_BITMAP1, 31 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 31 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_31_63_31);
	} else {
		rectfill(&RGB565_BITMAP1, 31 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 31 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_00_00_31);
	}

	if (status_led & 0x00000008) {
		rectfill(&RGB565_BITMAP1, 32 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 32 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_31_63_31);
	} else {
		rectfill(&RGB565_BITMAP1, 32 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 32 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_00_00_31);
	}

	if (status_led & 0x00000004) {
		rectfill(&RGB565_BITMAP1, 33 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 33 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_31_63_31);
	} else {
		rectfill(&RGB565_BITMAP1, 33 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 33 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_00_00_31);
	}

	if (status_led & 0x00000002) {
		rectfill(&RGB565_BITMAP1, 34 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 34 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_31_63_31);
	} else {
		rectfill(&RGB565_BITMAP1, 34 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 34 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_00_00_31);
	}

	if (status_led & 0x00000001) {
		rectfill(&RGB565_BITMAP1, 35 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 35 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_31_63_31);
	} else {
		rectfill(&RGB565_BITMAP1, 35 * RGB565_CHAR_WIDTH,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 35 * RGB565_CHAR_WIDTH + 6,
				RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_00_00_31);
	}
#endif
}

void draw_clean_led(void) {
#if PRINT_RGB
	rectfill(&RGB565_BITMAP1, 28 * RGB565_CHAR_WIDTH,
			RGB565_Y - 17 * RGB565_CHAR_HEIGHT, 35 * RGB565_CHAR_WIDTH + 6,
			RGB565_Y - 17 * RGB565_CHAR_HEIGHT + 10, RGB565_08_16_08);
#endif
}

void draw_logo(void) {
#if PRINT_RGB

	// Initialize display RGB565_CHAR1 with UTIA Logo in RGB565_CHARUTIA
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1b, 0, 0, 0, 0,
			RGB565_X, RGB565_Y, RGB565_00_00_31, RGB565_08_16_08);
	RGB565_CHAR_Border(&RGB565_BITMAP1, &RGB565_CHAR1b);
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1, 0 + 3, 0 + 3, 0 + 3,
			0 + 3, RGB565_X - 2, RGB565_Y - 3, RGB565_31_63_31, RGB565_08_16_08);
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHARUTIAb,
			RGB565_CHAR_WIDTH, RGB565_Y - 7 * RGB565_CHAR_HEIGHT,
			RGB565_CHAR_WIDTH, RGB565_Y - 7 * RGB565_CHAR_HEIGHT,
			18 * RGB565_CHAR_WIDTH + RGB565_CHAR_WIDTH,
			RGB565_Y - RGB565_CHAR_HEIGHT, RGB565_00_00_31, RGB565_08_16_08);
	RGB565_CHAR_Border(&RGB565_BITMAP1, &RGB565_CHARUTIAb);
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHARUTIA,
			2 * RGB565_CHAR_WIDTH, RGB565_Y - 7 * RGB565_CHAR_HEIGHT,
			2 * RGB565_CHAR_WIDTH, RGB565_Y - 7 * RGB565_CHAR_HEIGHT,
			18 * RGB565_CHAR_WIDTH, RGB565_Y - 2 * RGB565_CHAR_HEIGHT,
			RGB565_00_00_31, RGB565_08_16_08);
	RGB565_CHAR_LogoUTIA(&RGB565_BITMAP1, &RGB565_CHARUTIA);

	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 10, 15);
	RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1,
			UTIASPEDK_RGB565_XGA_TXT);
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 10, 30);
	RGB565_CHAR_WriteString(
			&RGB565_BITMAP1,
			&RGB565_CHAR1,
			" (c) 2014 UTIA AV CR v.v.i.   http://www.utia.cas.cz   Contact: Jiri Kadlec email: kadlec@utia.cas.cz");

#endif
}

void draw_fpga(unsigned int capabilities1) {
#if PRINT_RGB
#if UTIASPEDK_FPGA_3S700AN
#else
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 20 * RGB565_CHAR_WIDTH,
			RGB565_Y - 17 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1, "FPU");
#endif

	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 24 * RGB565_CHAR_WIDTH,
			RGB565_Y - 17 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1, "CPU");

	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 20 * RGB565_CHAR_WIDTH,
			RGB565_Y - 10 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1, "MPMC");

	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 20 * RGB565_CHAR_WIDTH,
			RGB565_Y - 3 * RGB565_CHAR_HEIGHT - 1);
	RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1, "VGA");

	//RGB565_CHAR_SetPos (&RGB565_BITMAP1,&RGB565_CHAR1,2*RGB565_CHAR_WIDTH,RGB565_Y- 12*RGB565_CHAR_HEIGHT);
	//RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1, "Core Power:");
	//RGB565_CHAR_SetPos (&RGB565_BITMAP1,&RGB565_CHAR1,2*RGB565_CHAR_WIDTH,RGB565_Y- 10*RGB565_CHAR_HEIGHT);
	//RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1, "           ");

	//Write Type of the Accelerator on Screen.
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 37 * RGB565_CHAR_WIDTH,
			RGB565_Y - 14 * RGB565_CHAR_HEIGHT);
	if (capabilities1 == 0x3FFF) {
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1,
				"fp11_1x1_v1_1x_a");
	}
	if (capabilities1 == 0x1BFFF) {
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1,
				"fp11_1x1_v1_2x_a");
	}
	if (capabilities1 == 0x1FFFF) {
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1,
				"fp11_1x1_v1_3x_a");
	}
	if (capabilities1 == 0x103FFF) {
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1,
				"fp12_1x1_v1_1x_a");
	}
	if (capabilities1 == 0x11BFFF) {
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1,
				"fp12_1x1_v1_2x_a");
	}
	if (capabilities1 == 0x11FFFF) {
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1,
				"fp12_1x1_v1_3x_a");
	}

	// FPGA box
	rect(&RGB565_BITMAP1, 20 * RGB565_CHAR_WIDTH - 4,
			RGB565_Y - 16 * RGB565_CHAR_HEIGHT + 8, 36 * RGB565_CHAR_WIDTH - 2,
			RGB565_Y - 2 * RGB565_CHAR_HEIGHT - 1, RGB565_31_63_31);
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 20 * RGB565_CHAR_WIDTH,
			RGB565_Y - 2 * RGB565_CHAR_HEIGHT + 2);
	RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1, UTIASPEDK_FPGA_TXT);

	// MB
	rect(&RGB565_BITMAP1, 24 * RGB565_CHAR_WIDTH,
			RGB565_Y - 15 * RGB565_CHAR_HEIGHT, 27 * RGB565_CHAR_WIDTH - 1,
			RGB565_Y - 3 * RGB565_CHAR_HEIGHT, RGB565_31_63_31);
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 25 * RGB565_CHAR_WIDTH,
			RGB565_Y - 14 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "M",
			RGB565_31_63_31, RGB565_08_16_08);
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 25 * RGB565_CHAR_WIDTH,
			RGB565_Y - 13 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "i",
			RGB565_31_63_31, RGB565_08_16_08);
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 25 * RGB565_CHAR_WIDTH,
			RGB565_Y - 12 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "c",
			RGB565_31_63_31, RGB565_08_16_08);
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 25 * RGB565_CHAR_WIDTH,
			RGB565_Y - 11 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "r",
			RGB565_31_63_31, RGB565_08_16_08);
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 25 * RGB565_CHAR_WIDTH,
			RGB565_Y - 10 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "o",
			RGB565_31_63_31, RGB565_08_16_08);
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 25 * RGB565_CHAR_WIDTH,
			RGB565_Y - 9 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "b",
			RGB565_31_63_31, RGB565_08_16_08);
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 25 * RGB565_CHAR_WIDTH,
			RGB565_Y - 8 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "l",
			RGB565_31_63_31, RGB565_08_16_08);
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 25 * RGB565_CHAR_WIDTH,
			RGB565_Y - 7 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "a",
			RGB565_31_63_31, RGB565_08_16_08);
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 25 * RGB565_CHAR_WIDTH,
			RGB565_Y - 6 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "z",
			RGB565_31_63_31, RGB565_08_16_08);
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 25 * RGB565_CHAR_WIDTH,
			RGB565_Y - 5 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "e",
			RGB565_31_63_31, RGB565_08_16_08);

	// eFLASH
	rect(&RGB565_BITMAP1, (24+4) * RGB565_CHAR_WIDTH,
			RGB565_Y - 11 * RGB565_CHAR_HEIGHT, (27+4) * RGB565_CHAR_WIDTH - 1,
			RGB565_Y - 3 * RGB565_CHAR_HEIGHT, RGB565_31_63_31 );
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, (25+4) * RGB565_CHAR_WIDTH,
			RGB565_Y - 10 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "e",
			RGB565_31_63_31, RGB565_08_16_08 );
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, (25+4) * RGB565_CHAR_WIDTH,
			RGB565_Y - 9 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "F",
			RGB565_31_63_31, RGB565_08_16_08 );
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, (25+4) * RGB565_CHAR_WIDTH,
			RGB565_Y - 8 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "L",
			RGB565_31_63_31, RGB565_08_16_08 );
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, (25+4) * RGB565_CHAR_WIDTH,
			RGB565_Y - 7 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "A",
			RGB565_31_63_31, RGB565_08_16_08 );
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, (25+4) * RGB565_CHAR_WIDTH,
			RGB565_Y - 6 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "S",
			RGB565_31_63_31, RGB565_08_16_08 );
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, (25+4) * RGB565_CHAR_WIDTH,
			RGB565_Y - 5 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "H",
			RGB565_31_63_31, RGB565_08_16_08 );

	// NVRAM
	rect(&RGB565_BITMAP1, (24+8) * RGB565_CHAR_WIDTH,
			RGB565_Y - 11 * RGB565_CHAR_HEIGHT, (27+8) * RGB565_CHAR_WIDTH - 1,
			RGB565_Y - 3 * RGB565_CHAR_HEIGHT, RGB565_31_63_31 );
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, (25+8) * RGB565_CHAR_WIDTH,
			RGB565_Y - 10 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "N",
			RGB565_31_63_31, RGB565_08_16_08 );
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, (25+8) * RGB565_CHAR_WIDTH,
			RGB565_Y - 9 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "V",
			RGB565_31_63_31, RGB565_08_16_08 );
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, (25+8) * RGB565_CHAR_WIDTH,
			RGB565_Y - 8 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, " ",
			RGB565_31_63_31, RGB565_08_16_08 );
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, (25+8) * RGB565_CHAR_WIDTH,
			RGB565_Y - 7 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "R",
			RGB565_31_63_31, RGB565_08_16_08 );
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, (25+8) * RGB565_CHAR_WIDTH,
			RGB565_Y - 6 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "A",
			RGB565_31_63_31, RGB565_08_16_08 );
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, (25+8) * RGB565_CHAR_WIDTH,
			RGB565_Y - 5 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteColorString(&RGB565_BITMAP1, &RGB565_CHAR1, "M",
			RGB565_31_63_31, RGB565_08_16_08 );


	// MPMC
	rectfill(&RGB565_BITMAP1, 20 * RGB565_CHAR_WIDTH,
			RGB565_Y - 9 * RGB565_CHAR_HEIGHT + 2, 23 * RGB565_CHAR_WIDTH,
			RGB565_Y - 6 * RGB565_CHAR_HEIGHT - 2, RGB565_31_63_31);
	rectfill(&RGB565_BITMAP1, 23 * RGB565_CHAR_WIDTH,
			RGB565_Y - 8 * RGB565_CHAR_HEIGHT, 24 * RGB565_CHAR_WIDTH,
			RGB565_Y - 7 * RGB565_CHAR_HEIGHT, RGB565_31_63_31);

	// VGA
	rectfill(&RGB565_BITMAP1, 21 * RGB565_CHAR_WIDTH,
			RGB565_Y - 6 * RGB565_CHAR_HEIGHT + 2, 22 * RGB565_CHAR_WIDTH,
			RGB565_Y - 6 * RGB565_CHAR_HEIGHT - 2, RGB565_31_63_31);
	rectfill(&RGB565_BITMAP1, 20 * RGB565_CHAR_WIDTH,
			RGB565_Y - 6 * RGB565_CHAR_HEIGHT + 2, 23 * RGB565_CHAR_WIDTH,
			RGB565_Y - 3 * RGB565_CHAR_HEIGHT - 2, RGB565_31_63_31);

	// FPU
#if UTIASPEDK_FPGA_3S700AN
#else
	rectfill(&RGB565_BITMAP1, 20 * RGB565_CHAR_WIDTH,
			RGB565_Y - 15 * RGB565_CHAR_HEIGHT, 23 * RGB565_CHAR_WIDTH,
			RGB565_Y - 12 * RGB565_CHAR_HEIGHT, RGB565_03_07_03);
	rectfill(&RGB565_BITMAP1, 23 * RGB565_CHAR_WIDTH,
			RGB565_Y - 14 * RGB565_CHAR_HEIGHT, 24 * RGB565_CHAR_WIDTH,
			RGB565_Y - 13 * RGB565_CHAR_HEIGHT, RGB565_03_07_03);
#endif

	//accelertor
	draw_accelerator_off();
#endif
}

void draw_nop_mw(void) {
#if PRINT_RGB
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 2 * RGB565_CHAR_WIDTH,
			RGB565_Y - 10 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1, "       ");
#endif
}

void draw_gr_mw(void) {
#if PRINT_RGB
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 2 * RGB565_CHAR_WIDTH,
			RGB565_Y - 10 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1, "        ");
#endif
}

void draw_op_mw(void) {
#if PRINT_RGB
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 2 * RGB565_CHAR_WIDTH,
			RGB565_Y - 10 * RGB565_CHAR_HEIGHT);
	RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1, "        ");
#endif
}

void draw_font(void) {
#if PRINT_RGB
	int k;
	RGB565_CHAR_SetPos(&RGB565_BITMAP1, &RGB565_CHAR1, 10, 108);
	for (k = 32; k < 128; k++) {
		RGB565_CHAR_Write(&RGB565_BITMAP1, &RGB565_CHAR1, (unsigned char) k);
	}
#endif
}

void draw_color_map(void) {
#if PRINT_RGB
	int i, j, k, red, green, blue;
	pixel_t clr;
	// Plot color map
	for (k = 0; k < 30; k = k + 10) {
		for (i = 0; i < 32; i++) {
			j = 20 * i;
			red = 0;
			green = 0;
			blue = 0;
			if (k == 0)
				blue = i;
			if (k == 10)
				green = i + i;
			if (k == 20)
				red = i;
			clr = (pixel_t) (red << 11 | green << 5 | blue);
			rectfill(&RGB565_BITMAP1, j + 50, k + 50, j + 60, k + 60, clr);
			red = 0;
			green = 0;
			blue = 0;
			if (k == 0)
				blue = i;
			if (k == 10)
				green = i + i + 1;
			if (k == 20)
				red = i;
			clr = (pixel_t) (red << 11 | green << 5 | blue);
			rectfill(&RGB565_BITMAP1, j + 60, k + 50, j + 70, k + 60, clr);
		}
	}
#endif
}

void draw_circles_triangles(void) {
#if PRINT_RGB
	int i, j;
	static int rgb565c[8] = { (0 << 11 | 0 << 5 | 0), (0 << 11 | 0 << 5 | 31),
			(0 << 11 | 63 << 5 | 0), (0 << 11 | 63 << 5 | 31), (31 << 11
					| 0 << 5 | 0), (31 << 11 | 0 << 5 | 31), (31 << 11 | 63 << 5
					| 0), (31 << 11 | 63 << 5 | 31) };
	// Plot circles and triangles
	for (j = 0; j < 8; j++) {
		for (i = 1; i < (10 * RGB565_CHAR_HEIGHT); i++) {
			ellipsefill(&RGB565_BITMAP1, RGB565_X - 52 * RGB565_CHAR_WIDTH,
					RGB565_Y - 12 * RGB565_CHAR_HEIGHT, i, i, rgb565c[j]);
			line(&RGB565_BITMAP1, RGB565_X - 32 * RGB565_CHAR_WIDTH,
					RGB565_Y - 2 * RGB565_CHAR_HEIGHT,
					RGB565_X - 2 * RGB565_CHAR_WIDTH,
					RGB565_Y - 2 * RGB565_CHAR_HEIGHT, rgb565c[j]);
			line(&RGB565_BITMAP1, RGB565_X - 32 * RGB565_CHAR_WIDTH,
					RGB565_Y - 2 * RGB565_CHAR_HEIGHT,
					RGB565_X - 17 * RGB565_CHAR_WIDTH,
					RGB565_Y - 2 * RGB565_CHAR_HEIGHT - 2 * i, rgb565c[j]);
			line(&RGB565_BITMAP1, RGB565_X - 2 * RGB565_CHAR_WIDTH,
					RGB565_Y - 2 * RGB565_CHAR_HEIGHT,
					RGB565_X - 17 * RGB565_CHAR_WIDTH,
					RGB565_Y - 2 * RGB565_CHAR_HEIGHT - 2 * i, rgb565c[j]);
		}
	}
	for (i = 1; i < (10 * RGB565_CHAR_HEIGHT); i++) {
		ellipsefill(&RGB565_BITMAP1, RGB565_X - 52 * RGB565_CHAR_WIDTH,
				RGB565_Y - 12 * RGB565_CHAR_HEIGHT, i, i, RGB565_08_16_08 );
		line(&RGB565_BITMAP1, RGB565_X - 32 * RGB565_CHAR_WIDTH,
				RGB565_Y - 2 * RGB565_CHAR_HEIGHT,
				RGB565_X - 2 * RGB565_CHAR_WIDTH,
				RGB565_Y - 2 * RGB565_CHAR_HEIGHT, RGB565_08_16_08 );
		line(&RGB565_BITMAP1, RGB565_X - 32 * RGB565_CHAR_WIDTH,
				RGB565_Y - 2 * RGB565_CHAR_HEIGHT,
				RGB565_X - 17 * RGB565_CHAR_WIDTH,
				RGB565_Y - 2 * RGB565_CHAR_HEIGHT - 2 * i, RGB565_08_16_08 );
		line(&RGB565_BITMAP1, RGB565_X - 2 * RGB565_CHAR_WIDTH,
				RGB565_Y - 2 * RGB565_CHAR_HEIGHT,
				RGB565_X - 17 * RGB565_CHAR_WIDTH,
				RGB565_Y - 2 * RGB565_CHAR_HEIGHT - 2 * i, RGB565_08_16_08 );
	}
#endif
}

void draw_display_0(void) {
#if PRINT_RGB
	// Initialize display RGB565_CHAR1_0
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1b_0, 0 + 8, 180 - 36,
			0 + 8, 180 - 36, 256 - 8, 360 - 36, RGB565_00_00_16,
			RGB565_08_16_08);
	RGB565_CHAR_Border(&RGB565_BITMAP1, &RGB565_CHAR1b_0);
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1_0, 0 + 8 + 3,
			180 - 36 + 3, 0 + 8 + 3, 180 - 36 + 3, 256 - 8 - 2, 360 - 36 - 3,
			RGB565_00_32_00, RGB565_03_07_03);
#endif
}

void draw_display_1(void) {
#if PRINT_RGB
	// Initialize display RGB565_CHAR1_1
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1b_1, 256 + 8,
			180 - 36, 256 + 8, 180 - 36, 512 - 8, 360 - 36, RGB565_00_00_16,
			RGB565_08_16_08);
	RGB565_CHAR_Border(&RGB565_BITMAP1, &RGB565_CHAR1b_1);
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1_1, 256 + 8 + 3,
			180 - 36 + 3, 256 + 8 + 3, 180 - 36 + 3, 512 - 8 - 2, 360 - 36 - 3,
			RGB565_00_32_00, RGB565_03_07_03);
#endif
}

void draw_display_2(void) {
#if PRINT_RGB
	// Initialize display RGB565_CHAR1_2
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1b_2, 512 + 8,
			180 - 36, 512 + 8, 180 - 36, 768 - 8, 360 - 36, RGB565_00_00_16,
			RGB565_08_16_08);
	RGB565_CHAR_Border(&RGB565_BITMAP1, &RGB565_CHAR1b_2);
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1_2, 512 + 3,
			180 - 36 + 3, 512 + 8 + 3, 180 - 36 + 3, 768 - 8 - 2, 360 - 36 - 3,
			RGB565_00_32_00, RGB565_03_07_03);
#endif
}

void draw_display_3(void) {
#if PRINT_RGB
	// Initialize display RGB565_CHAR1_3
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1b_3, 768 + 8,
			180 - 36, 768 + 8, 180 - 36, 1024 - 8, 360 - 36, RGB565_00_00_16,
			RGB565_08_16_08);
	RGB565_CHAR_Border(&RGB565_BITMAP1, &RGB565_CHAR1b_3);
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1_3, 768 + 8 + 3,
			180 - 36 + 3, 768 + 8 + 3, 180 - 36 + 3, 1024 - 8 - 2,
			360 - 36 - 3, RGB565_00_32_00, RGB565_03_07_03);
#endif
}

void draw_display_4(void) {
#if PRINT_RGB
	// Initialize display RGB565_CHAR1_4
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1b_4, 8, 372 - 36, 8,
			372 - 36, 256 - 8, 552 - 36, RGB565_00_00_16, RGB565_08_16_08);
	RGB565_CHAR_Border(&RGB565_BITMAP1, &RGB565_CHAR1b_4);
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1_4, 8 + 3,
			372 - 36 + 3, 8 + 3, 372 - 36 + 3, 256 - 8 - 2, 552 - 36 - 3,
			RGB565_00_32_00, RGB565_03_07_03);
#endif
}

void draw_display_5(void) {
#if PRINT_RGB
	// Initialize display RGB565_CHAR1_5
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1b_5, 256 + 8,
			372 - 36, 256 + 8, 372 - 36, 512 - 8, 552 - 36, RGB565_00_00_16,
			RGB565_08_16_08);
	RGB565_CHAR_Border(&RGB565_BITMAP1, &RGB565_CHAR1b_5);
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1_5, 256 + 8 + 3,
			372 - 36 + 3, 256 + 8 + 3, 372 - 36 + 3, 512 - 8 - 2, 552 - 36 - 3,
			RGB565_00_32_00, RGB565_03_07_03);
#endif
}

void draw_display_6(void) {
#if PRINT_RGB
	// Initialize display RGB565_CHAR1_6
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1b_6, 512 + 8,
			372 - 36, 512 + 8, 372 - 36, 768 - 8, 552 - 36, RGB565_00_00_16,
			RGB565_08_16_08);
	RGB565_CHAR_Border(&RGB565_BITMAP1, &RGB565_CHAR1b_6);
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1_6, 512 + 8 + 3,
			372 - 36 + 3, 512 + 8 + 3, 372 - 36 + 3, 768 - 8 - 2, 552 - 36 - 3,
			RGB565_00_32_00, RGB565_03_07_03);
#endif
}

void draw_display_7(void) {
#if PRINT_RGB
	// Initialize display RGB565_CHAR1_7
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1b_7, 768 + 8,
			372 - 36, 768 + 8, 372 - 36, 1024 - 8, 552 - 36, RGB565_00_00_16,
			RGB565_08_16_08);
	RGB565_CHAR_Border(&RGB565_BITMAP1, &RGB565_CHAR1b_7);
	RGB565_CHAR_Initialize(&RGB565_BITMAP1, &RGB565_CHAR1_7, 768 + 8 + 3,
			372 - 36 + 3, 768 + 8 + 3, 372 - 36 + 3, 1024 - 8 - 2,
			552 - 36 - 3, RGB565_00_32_00, RGB565_03_07_03);
#endif
}

void draw_display_11(void) {
#if PRINT_RGB
	// Initialize display RGB565_CHAR1_11
	//RGB565_CHAR_Initialize (&RGB565_BITMAP1,&RGB565_CHAR1b_11,768 ,564+24 ,768+8 ,564+24 ,1024-8 ,744+12 ,RGB565_00_00_16,RGB565_08_16_08);
	//RGB565_CHAR_Border (&RGB565_BITMAP1,&RGB565_CHAR1b_11);
	//RGB565_CHAR_Initialize (&RGB565_BITMAP1,&RGB565_CHAR1_11, 768+3,564+24+3,768+8+3,564+24+3,1024-8-2,744+12-3,RGB565_00_32_00,RGB565_03_07_03);
#endif
}

void draw_display_0_1_2_3_text(void) {
#if PRINT_OUTPUT
	xil_printf("\r\n Write description text ... ");
#endif

#if PRINT_RGB
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_0,"\r\n Plot Circles and Triangles:");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_0,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_0,"\r\n MicroBlaze program writes");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_0,"\r\n to video frame buffer.");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_0,"\r\n MPMC RGB565 16 bit/pixel");

	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_1,"\r\n Supported boards:");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_1,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_1,"\r\n Xilinx 6SLX45T-3 106 MHz");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_1,"\r\n Xilinx 6SLX16-2   83 MHz");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_1,"\r\n Xilinx 3SD1800a-4 75 MHz");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_1,"\r\n Xilinx 3S700an-4  75 MHz");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_1,"\r\n Xilinx 3S1600e-4  75 MHz");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_1,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_1,"\r\n See:");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_1,"\r\n http://sp.utia.cas.cz");

	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_2,"\r\n Supported Design Flows:");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_2,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_2,"\r\n Xilinx ISE/EDK 13.2");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_2,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_2,"\r\n Supported SOC:");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_2,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_2,"\r\n MicroBlaze PLB   8.20.a");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_2,"\r\n Mpmc             6.04.a");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_2,"\r\n Xps_EthernetLite 4.00.a");

	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_3,"\r\n Supported Operating System:");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_3,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_3,"\r\n Xilinx:");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_3,"\r\n standalone 13.2");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_3,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_3,"\r\n PetaLogix: ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_3,"\r\n uCLinux, no MMU");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_3,"\r\n petalinux-v0.40-final");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_3,"\r\n http://www.petalogix.com");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_3,"\r\n ");
#endif

#if PRINT_OUTPUT
	xil_printf("\r\n Done.");
#endif
}

void draw_display_4_5_6_7_text(void) {

#if PRINT_RGB
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_4,"\r\n Test of FP Accelerator:");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_4,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_4,"\r\n MicroBlaze is testing");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_4,"\r\n all floating point");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_4,"\r\n vector operations");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_4,"\r\n of HW accelerator.");

	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_5,"\r\n Supported FP Accelerators:");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_5,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_5,"\r\n bce_fp11_1x1_v1_1x_a");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_5,"\r\n bce_fp11_1x1_v1_2x_a");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_5,"\r\n bce_fp11_1x1_v1_3x_a");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_5,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_5,"\r\n Supported vector op:");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_5,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_5,"\r\n bce_fp11_1x1_v1_1x_a");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_5,"\r\n basic=copy,add,sub,mul");

	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_6,"\r\n Supported vector op:");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_6,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_6,"\r\n bce_fp11_1x1_v1_2x_a");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_6,"\r\n basic,mac");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_6,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_6,"\r\n bce_fp11_1x1_v1_3x_a");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_6,"\r\n basic,mac,dotprod");

	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_7,"\r\n Performance example:");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_7,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_7,"\r\n Xilinx 3S1600e-4");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_7,"\r\n Clock  75 MHz");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_7,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_7,"\r\n bce_fp11_1x1_v1_3x_a");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_7,"\r\n ");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_7,"\r\n FIR filter: 100 MFLOPS");
	//	RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_7,"\r\n LMS filter:  60 MFLOPS");
#endif
}

float vprod_mb(unsigned int nn, float * afmb, float * bfmb) {
	unsigned int i, j;
	float zs[4];

	for (i = 0; i < 4; i++) {
		zs[i] = 0;
	}

	j = 0;
	for (i = 0; i < nn; i++) {
		zs[j] += afmb[i] * bfmb[i];
		if (j < 3)
			j = j + 1;
		else
			j = 0;
	}
	zs[0] += zs[1];
	zs[2] += zs[3];
	zs[0] += zs[2];

	return ((float) zs[0]); //vector product
}

void vmac_mb(unsigned int nn, float * afmb, float * bfmb, float * zfmb) {
	unsigned int i;
	for (i = 0; i < nn; i++) {
		zfmb[i] += afmb[i] * bfmb[i];
	}
}

void vmsubac_mb(unsigned int nn, float * afmb, float * bfmb, float * zfmb) {
	unsigned int i;
	for (i = 0; i < nn; i++) {
		zfmb[i] -= afmb[i] * bfmb[i];
	}
}

//////////////////////////////////////////////////////////////////////////
// DSP_PLATFORM (c) UTIA AV CR v.v.i. 2007-2010
//

void bce_fp11_1x1_0_VZ2A(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {

	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;

	//====================================================
	if (capab & WAL_BCE_JK_CAP_VZ2A) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VZ2A '%s' .............", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VZ2A .............");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VZ2A;
		for (i = 0; i < nn; i++) {
			zf1[i] = (float) i + (float) 0.123456;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			if (af1r[i] != zf1[i]) {
#if PRINT_OUTPUT
				xil_printf("\r\n VZ2A Error: af1r[%d] = %d", i,
						(unsigned int) af1r[i]);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VZ2A is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VZ2A is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VB2A(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {

	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VB2A) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VB2A '%s' .............", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VB2A .............");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VB2A;
		for (i = 0; i < nn; i++) {
			bf1[i] = (float) i + (float) 0.123456;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			if (af1r[i] != bf1[i]) {
#if PRINT_OUTPUT
				xil_printf("\r\n VB2A Error: af1r[%d] = %d", i,
						(unsigned int) af1r[i]);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VB2A is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VB2A is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VZ2B(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {

	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VZ2B) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VZ2B '%s' .............", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VZ2B .............");
#endif

		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VZ2B;
		for (i = 0; i < nn; i++) {
			zf1[i] = (float) i + (float) 1.23456;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			if (bf1r[i] != zf1[i]) {
#if PRINT_OUTPUT
				xil_printf("\r\n VZ2B Error: bf1r[%d] = %d", i,
						(unsigned int) bf1r[i]);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VZ2B is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VZ2B is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VA2B(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {

	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VA2B) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VA2B '%s' .............", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VA2B .............");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VA2B;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.23456;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			if (bf1r[i] != af1[i]) {
#if PRINT_OUTPUT
				xil_printf("\r\n VA2B Error: bf1r[%d] = %d", i,
						(unsigned int) bf1r[i]);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VA2B is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VA2B is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VADD(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VADD) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VADD '%s' .............", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VADD .............");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VADD;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 0.123456;
			bf1[i] = (float) i + (float) 0.789;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] + bf1[i];
			err = zf1r[i] - r;
			if (((float) err > (float) ERR) || ((float) err < (float) -ERR)) {
#if PRINT_OUTPUT
				xil_printf("\r\n VADD Error: zf1r[%d] = %d   r = %d", i,
						(unsigned int) zf1r[i], (unsigned int) r);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VADD is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VADD is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VADD_BZ2A(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VADD_BZ2A) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VADD_BZ2A '%s' ........", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VADD_BZ2A ........");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VADD_BZ2A;
		for (i = 0; i < nn; i++) {
			bf1[i] = (float) i + (float) 1.23456;
			zf1[i] = (float) i + (float) 1.789;
			af1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = bf1[i] + zf1[i];
			err = af1r[i] - r;
			if (((float) err > (float) ERR) || ((float) err < (float) -ERR)) {
#if PRINT_OUTPUT
				xil_printf("\r\n VADD_BZ2A Error: af1r[%d] = %d   r = %d", i,
						(unsigned int) af1r[i], (unsigned int) r);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VADD_BZ2A is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VADD_BZ2A is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VADD_AZ2B(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VADD_AZ2B) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VADD_AZ2B '%s' ........", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VADD_AZ2B ........");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VADD_AZ2B;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.12345;
			zf1[i] = (float) i + (float) 1.23456;
			bf1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] + zf1[i];
			err = bf1r[i] - r;
			if (((float) err > (float) ERR) || ((float) err < (float) -ERR)) {
#if PRINT_OUTPUT
				xil_printf("\r\n VADD_AZ2B Error: bf1r[%d] = %d   r = %d", i,
						(unsigned int) bf1r[i], (unsigned int) r);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VADD_AZ2B is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VADD_AZ2B is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VSUB(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VSUB) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VSUB '%s' .............", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VSUB .............");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VSUB;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 2.23456;
			bf1[i] = (float) 1.123456;
			zf1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] - bf1[i];
			err = zf1r[i] - r;
			if (((float) err > (float) ERR) || ((float) err < (float) -ERR)) {
#if PRINT_OUTPUT
				xil_printf("\r\n VSUB Error: zf1r[%d] = %d   r = %d", i,
						(unsigned int) zf1r[i], (unsigned int) r);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VSUB is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VSUB is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VSUB_BZ2A(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VSUB_BZ2A) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VSUB_BZ2A '%s' ........", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VSUB_BZ2A ........");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VSUB_BZ2A;
		for (i = 0; i < nn; i++) {
			bf1[i] = (float) i + (float) 2.12345;
			zf1[i] = (float) 1.6789;
			af1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = bf1[i] - zf1[i];
			err = af1r[i] - r;
			if (((float) err > (float) ERR) || ((float) err < (float) -ERR)) {
#if PRINT_OUTPUT
				xil_printf("\r\n VSUB_BZ2A Error: af1r[%d] = %d   r = %d", i,
						(unsigned int) af1r[i], (unsigned int) r);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VSUB_BZ2A is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VSUB_BZ2A is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VSUB_AZ2B(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VSUB_AZ2B) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VSUB_AZ2B '%s' ........", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VSUB_AZ2B ........");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VSUB_AZ2B;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.123456;
			zf1[i] = (float) i + (float) 1.6789;
			bf1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] - zf1[i];
			err = bf1r[i] - r;
			if (((float) err > (float) ERR) || ((float) err < (float) -ERR)) {
#if PRINT_OUTPUT
				xil_printf("\r\n VSUB_AZ2B Error: bf1r[%d] = %d   r = %d", i,
						(unsigned int) bf1r[i], (unsigned int) r);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VSUB_AZ2B is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VSUB_AZ2B is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VMULT(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMULT) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VMULT '%s' ............", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VMULT ............");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMULT;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.12345;
			bf1[i] = (float) i + (float) 0.76543;
			zf1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] * bf1[i];
			err = zf1r[i] - r;
			if (((float) err > (float) ERR) || ((float) err < (float) -ERR)) {
#if PRINT_OUTPUT
				xil_printf("\r\n VMULT Error: zf1r[%d] = %d   r = %d", i,
						(unsigned int) zf1r[i], (unsigned int) r);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VMULT is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VMULT is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VMULT_BZ2A(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMULT_BZ2A) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VMULT_BZ2A '%s' .......", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VMULT_BZ2A .......");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMULT_BZ2A;
		for (i = 0; i < nn; i++) {
			bf1[i] = (float) i + (float) 1.23456;
			zf1[i] = (float) i + (float) 0.76543;
			af1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = bf1[i] * zf1[i];
			err = af1r[i] - r;
			if (((float) err > (float) ERR) || ((float) err < (float) -ERR)) {
#if PRINT_OUTPUT
				xil_printf("\r\n VMULT_BZ2A Error: af1r[%d] = %d   r = %d", i,
						(unsigned int) af1r[i], (unsigned int) r);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VMULT_BZ2A is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VMULT_BZ2A is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VMULT_AZ2B(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMULT_AZ2B) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VMULT_AZ2B '%s' .......", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VMULT_AZ2B .......");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMULT_AZ2B;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.12345;
			zf1[i] = (float) i + (float) 0.76543;
			bf1r[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] * zf1[i];
			err = bf1r[i] - r;
			if (((float) err > (float) ERR) || ((float) err < (float) -ERR)) {
#if PRINT_OUTPUT
				xil_printf("\r\n VMULT_AZ2B Error: bf1r[%d] = %d   r = %d", i,
						(unsigned int) bf1r[i], (unsigned int) r);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VMULT_AZ2B is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VMULT_AZ2B is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VPROD(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VPROD) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VPROD '%s' ............", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VPROD ............");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 0;
		op = WAL_BCE_JK_VPROD;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.123456;
			bf1[i] = (float) i + (float) 0.76543;
			zf1[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		j = 0;
		r = vprod_mb(nn, af1, bf1);
		err = zf1r[0] - r;
		if (((float) err > (float) ERR) || ((float) err < (float) -ERR)) {
#if PRINT_OUTPUT
			xil_printf("\r\n VPROD Error: zf1r[0] = %d   r = %d",
					(unsigned int) zf1r[0], (unsigned int) r);
#endif
			j++;
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VPROD is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VPROD is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VMAC(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMAC) {
		if (NN > 10) {
			nn = 10;
		} else {
			nn = NN;
		}
#if PRINT_OUTPUT

		xil_printf("\r\n Test VMAC '%s' .............", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VMAC .............");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMAC;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.123456;
			bf1[i] = (float) i + (float) 0.76543;
			zf1[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		vmac_mb(nn, af1, bf1, zf1);

		j = 0;
		for (i = 0; i < nn; i++) {
			err = zf1[i] - zf1r[i];
			if (((float) err > (float) ERR) || ((float) err < (float) -ERR)) {
#if PRINT_OUTPUT
				xil_printf("\r\n VMAC Error: zf1[%d] = %d   zf1r[%d] = %d", i,
						(unsigned int) zf1[i], i, (unsigned int) zf1r[i]);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VMAC is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VMAC is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VMSUBAC(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMSUBAC) {
		if (NN > 10) {
			nn = 10;
		} else {
			nn = NN;
		}
#if PRINT_OUTPUT

		xil_printf("\r\n Test VMSUBAC '%s' ..........", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VMSUBAC ..........");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMSUBAC;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 1.123456;
			bf1[i] = (float) i + (float) 0.76543;
			zf1[i] = (float) 0.0;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		vmsubac_mb(nn, af1, bf1, zf1);

		j = 0;
		for (i = 0; i < nn; i++) {
			err = zf1[i] - zf1r[i];
			if (((float) err > (float) ERR) || ((float) err < (float) -ERR)) {
#if PRINT_OUTPUT
				xil_printf("\r\n VMSUBAC Error: zf1[%d] = %d   zf1r[%d] = %d",
						i, (unsigned int) zf1[i], i, (unsigned int) zf1r[i]);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VMSUBAC is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VMSUBAC is not supported.");
#endif
	}
	//====================================================
}

void bce_fp11_1x1_0_VDIV(wal_worker_t *wrk, int fw, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VDIV) {
		nn = NN;
#if PRINT_OUTPUT

		xil_printf("\r\n Test VDIV '%s' .............", wrk->name);
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n Test VDIV .............");
#endif
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VDIV;
		for (i = 0; i < nn; i++) {
			af1[i] = (float) i + (float) 0.123456;
			bf1[i] = (float) i + (float) 0.789;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af1[0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf1[0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk, fw);
		wal_bce_jk_sync_operation(wrk);

		wal_dmem2mb(wrk, 0, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf1r[0], nn);

		j = 0;
		for (i = 0; i < nn; i++) {
			r = af1[i] / bf1[i];
			err = zf1r[i] - r;
			if (((float) err > (float) ERR) || ((float) err < (float) -ERR)) {
#if PRINT_OUTPUT
				xil_printf("\r\n VDIV Error: zf1r[%d] = %d   r = %d", i,
						(unsigned int) zf1r[i], (unsigned int) r);
#endif
				j++;
			}
		}
		if (j == 0) {
#if PRINT_OUTPUT
			xil_printf(" OK");
#endif
#if PRINT_RGB
			RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0, " OK");
#endif
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n VDIV is not supported.");
#endif
#if PRINT_RGB
		RGB565_CHAR_WriteString(&RGB565_BITMAP1, &RGB565_CHAR1_0,
				"\r\n VDIV is not supported.");
#endif
	}
	//====================================================
}

void update_firmware(char * file_name, int fr, unsigned int * program_store) {

	int i, j, k, kk, kkk;
	char data[10];
	long length;

	fr = mfs_file_open(file_name, MFS_MODE_READ);
	if (fr != -1) {
#if PRINT_OUTPUT
		xil_printf("\r\n Updating firmware *s", file_name);
#endif
		length = mfs_file_lseek(fr, 0, MFS_SEEK_END);
		kkk = 0;
		for (k = 0; k < 3; k++) {
			for (kk = 0; kk < 10; kk++) {
				if (kkk < length) {
					mfs_file_read(fr, &data[kk], 1);
					kkk++;
				}
				if (data[kk] == 0x0A) {
					break;
				}
			}
			j = 0;
			for (i = 0; i <= kk; i++) {
				if ((data[i] >= 0x30) && (data[i] <= 0x39) && (kkk <= length)) {
					j = 10 * j + (data[i] - 0x30);
				}
			}
			program_store[k] = j;
		}
		for (k = EDKDSP_BOOT; k < 1024; k++) {
			for (kk = 0; kk < 10; kk++) {
				if (kkk < length) {
					mfs_file_read(fr, &data[kk], 1);
					kkk++;
				}
				if (data[kk] == 0x0A) {
					break;
				}
			}
			j = 0;
			for (i = 0; i <= kk; i++) {
				if ((data[i] >= 0x30) && (data[i] <= 0x39) && (kkk <= length)) {
					j = 10 * j + (data[i] - 0x30);
				}
			}
			program_store[k] = j;
		}
	} else {
#if PRINT_OUTPUT
		xil_printf("\r\n File %s not found.", file_name);
		xil_printf("\r\n Default firmware will be used.");
#endif
	}
	mfs_file_close(fr);
}

void platform_init_gpios() {
#ifndef NO_GPIOS
	/* set led gpio data direction to output */
	*(volatile unsigned int*) (LED_BASE + 4) = 0;

	/* set dip switch gpio data direction to in */
	*(volatile unsigned int*) (DIP_BASE + 4) = ~0;

	/* initialize leds to OFF */
	*(volatile int *) (LED_BASE) = 0;
#endif

	if (wal_init_worker(worker1)) {
#if PRINT_OUTPUT
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
#endif
		return;
	}

#if PRINT_RGB
	RGB565_BITMAP_Initialize(&RGB565_BITMAP1, (void*) VRAM_ADDR1, RGB565_X,
			RGB565_Y);
#endif

	draw_logo();
	draw_nop_mw();
	draw_gr_mw();
	draw_nop_mw();
	draw_font();
	draw_color_map();
//	jpg_display("images/2.jpg");
//	draw_circles_triangles();
	jpg_display("images/1.jpg");

	draw_display_0();
	draw_display_1();
	draw_display_2();
	draw_display_3();
	draw_display_4();
	draw_display_5();
	draw_display_6();
	draw_display_7();
}

int toggle_leds() {
	unsigned int ah, bh, zh, i;
	unsigned int capabilities1;
	unsigned int license1;
	int fr, fw;
	int num_blocks_used;
	int num_blocks_free;

	static int state = 1;

	if (state == 1) {
		state = 0;
	} else {
		state = ~state;
#ifndef NO_GPIOS
		*(volatile int *) (LED_BASE) = state;
#endif

#if PRINT_OUTPUT
		xil_printf("\r\n Tests of vector operations.");
#endif
		///////////////////////////////////////////////
		// Upgrade firmware
		for (i = 0; i < 3; i++) {
			fill_FP1101P0_program_store[i] = fill_FA1101P0_program_store[i];
			fill_FP1101P1_program_store[i] = fill_FA1101P1_program_store[i];
		}
		for (i = EDKDSP_BOOT; i < 1024; i++) {
			fill_FP1101P0_program_store[i] = fill_FA1101P0_program_store[i];
			fill_FP1101P1_program_store[i] = fill_FA1101P1_program_store[i];
		}



		fr = -1;
		update_firmware("FP1101P0.DEC", fr, fill_FP1101P0_program_store);
		mfs_delete_file("FP1101P0.DEC");

		update_firmware("FP1101P1.DEC", fr, fill_FP1101P1_program_store);
		mfs_delete_file("FP1101P1.DEC");

		mfs_delete_file("FP1101.TXT");

		fw = mfs_file_open("FP1101.TXT", MFS_MODE_CREATE);
		if (fw != -1) {
#if PRINT_OUTPUT
			xil_printf("\r\n File FP1101.TXT created for wr");
#endif
		} else {
#if PRINT_OUTPUT
			xil_printf("\r\n Failed to create File FP1101.TXT open for wr.");
#endif
		}

		//Program and initalize the accelerator HW. Get HW Operations Code.
		wal_set_firmware(worker1, WAL_PBID_P0, fill_FP1101P0_program_store, -1);
		wal_set_firmware(worker1, WAL_PBID_P1, fill_FP1101P1_program_store, -1);
		wal_get_capabilities(worker1, WAL_PBID_P0, &capabilities1);
#if PRINT_OUTPUT
		xil_printf("\r\n PB: HW Operations: Code = %x", capabilities1);
#endif

		draw_fpga(capabilities1);
		draw_fpu_on();
		draw_accelerator_green_on();
		draw_op_mw();
		draw_display_0();
		draw_display_1();
		draw_display_2();
		draw_display_3();
		draw_display_4();
		draw_display_5();
		draw_display_6();
		draw_display_7();

		for (ah = 0; ah < 2; ah++) {
			for (bh = 0; bh < 2; bh++) {
				for (zh = 0; zh < 2; zh++) {
					// Test of the HW evaluation license
					wal_get_license(worker1, WAL_PBID_P0, &license1);
					if (license1) {
#if PRINT_OUTPUT
						xil_printf("\r\n\r\n ah=%d  bh=%d  zh=%d", ah, bh, zh);
#endif
						bce_fp11_1x1_0_VZ2A(worker1, fw, capabilities1, ah, bh,
								zh);
						bce_fp11_1x1_0_VB2A(worker1, fw, capabilities1, ah, bh,
								zh);
						bce_fp11_1x1_0_VZ2B(worker1, fw, capabilities1, ah, bh,
								zh);
						bce_fp11_1x1_0_VA2B(worker1, fw, capabilities1, ah, bh,
								zh);
						bce_fp11_1x1_0_VADD(worker1, fw, capabilities1, ah, bh,
								zh);
						bce_fp11_1x1_0_VADD_BZ2A(worker1, fw, capabilities1,
								ah, bh, zh);
						bce_fp11_1x1_0_VADD_AZ2B(worker1, fw, capabilities1,
								ah, bh, zh);
						bce_fp11_1x1_0_VSUB(worker1, fw, capabilities1, ah, bh,
								zh);
						bce_fp11_1x1_0_VSUB_BZ2A(worker1, fw, capabilities1,
								ah, bh, zh);
						bce_fp11_1x1_0_VSUB_AZ2B(worker1, fw, capabilities1,
								ah, bh, zh);
						bce_fp11_1x1_0_VMULT(worker1, fw, capabilities1, ah,
								bh, zh);
						bce_fp11_1x1_0_VMULT_BZ2A(worker1, fw, capabilities1,
								ah, bh, zh);
						bce_fp11_1x1_0_VMULT_AZ2B(worker1, fw, capabilities1,
								ah, bh, zh);
						bce_fp11_1x1_0_VPROD(worker1, fw, capabilities1, ah,
								bh, zh);
						bce_fp11_1x1_0_VMAC(worker1, fw, capabilities1, ah, bh,
								zh);
						bce_fp11_1x1_0_VMSUBAC(worker1, fw, capabilities1, ah,
								bh, zh);
						//bce_fp11_1x1_0_VDIV(worker1, fw, capabilities1, ah, bh, zh);
					} else {
#if PRINT_OUTPUT
						xil_printf(
								"\r\n\r\n Evaluation license will expire soon. Please contact:");
						xil_printf(
								"\r\n UTIA AV CR, v.v.i.; Department of Signal Processing");
						xil_printf("\r\n Pod vodarenskou vezi 4");
						xil_printf("\r\n 18208, Prague 8, Czech republic");
						xil_printf("\r\n http://zs.utia.cas.cz \r\n");
#endif
#if PRINT_RGB
						RGB565_CHAR_WriteString(&RGB565_BITMAP1,
								&RGB565_CHAR1_0,
								"\r\n Evaluation license will expire soon. Please contact:");
						RGB565_CHAR_WriteString(&RGB565_BITMAP1,
								&RGB565_CHAR1_0,
								"\r\n UTIA AV CR, v.v.i.; Department of Signal Processing");
						RGB565_CHAR_WriteString(&RGB565_BITMAP1,
								&RGB565_CHAR1_0, "\r\n Pod vodarenskou vezi 4");
						RGB565_CHAR_WriteString(&RGB565_BITMAP1,
								&RGB565_CHAR1_0,
								"\r\n 18208, Prague 8, Czech republic");
						RGB565_CHAR_WriteString(&RGB565_BITMAP1,
								&RGB565_CHAR1_0,
								"\r\n http://zs.utia.cas.cz \r\n");
#endif
					}
				}
			}
		}
		draw_clean_led();
		draw_fpu_off();
		draw_accelerator_off();
		draw_nop_mw();

#if LINUX
#if PRINT_OUTPUT
		//xil_printf("\r\n\r\n Sleep 5 sec\r\n");
#endif
#if PRINT_RGB
		//RGB565_CHAR_WriteString (&RGB565_BITMAP1,&RGB565_CHAR1_0,"\r\n\r\n Sleep 5 sec");
#endif
		draw_nop_mw();
		//sleep(5);
#endif
		//}

		mfs_file_close(fw);
		mfs_get_usage(&num_blocks_used, &num_blocks_free);

		xil_printf("\r\nBlocks_used %d", num_blocks_used);
		xil_printf("\r\nBlocks_free %d", num_blocks_free);
		xil_printf("\r\n");

		mfs_ls();

	}
	return state;
}

unsigned int get_switch_state() {
	static int state_switch = 1;

	if (state_switch == 1) {
		state_switch = 0;
	} else {
		state_switch = ~state_switch;
		if (state_switch == 0) {
			jpg_display("images/1.jpg");
		} else {
			jpg_display("images/2.jpg");
		}
	}
#ifdef NO_GPIOS
	return 0x0;
#else
	return *(volatile unsigned int *) (DIP_BASE);
#endif

}
